'''
Software ExPI
Copyright Inria
Year 2021
Contact : wen.guo@inria.fr
GPL license.
'''
# AttModel_crossAtt_unshare.py

from torch.nn import Module
from torch import nn
import torch
import math
from model import GCN, XIA
import utils.util as util
import numpy as np
# from IPython import embed
from utils.opt import Options

from torch.autograd import Variable

class TimeDistributed(nn.Module):
    def __init__(self, module, batch_first=False):
        super(TimeDistributed, self).__init__()
        self.module = module
        self.batch_first = batch_first

    def forward(self, x):

        if len(x.size()) <= 2:
            return self.module(x)

        # Squash samples and timesteps into a single axis
        x_reshape = x.contiguous().view(-1, x.size(-1))  # (samples * timesteps, input_size)

        y = self.module(x_reshape)

        # We have to reshape Y
        if self.batch_first:
            y = y.contiguous().view(x.size(0), -1, y.size(-1))  # (samples, timesteps, output_size)
        else:
            y = y.view(-1, x.size(1), y.size(-1))  # (timesteps, samples, output_size)

        return y
    
class GuidClass(Module):
    def __init__(self):
        super(GuidClass, self).__init__()
        self.lstm1 = nn.LSTM(54,54,batch_first=True,num_layers=2)
        self.lstm2 = nn.LSTM(54,54,batch_first=True,num_layers=2)
        self.linear1 = nn.Linear(54*20,50)
        self.linear2 = nn.Linear(54*20,50)
        self.linear3 = nn.Linear(100,7)
        self.soft = nn.Softmax(dim=1)


    def forward(self,src):
        src1 = src[:,:,:54] # (bs, 50, 54)
        src2 = src[:,:,54:108] # (bs, 50, 54)
        for1, _ = self.lstm1(src1)
        for2, _ = self.lstm2(src2)
        x , y , z = for1.shape
        for1 = torch.reshape(for1,(x,y*z))
        for1 = self.linear1(for1)
        x , y , z = for2.shape
        for2 = torch.reshape(for2,(x,y*z))
        for2 = self.linear2(for2)
        for3 = torch.cat([for1,for2],axis=-1)
        for3 = self.linear3(for3)
        for3 = self.soft(for3)
        return for3
class AttModel(Module):

    def __init__(self, in_features=108, kernel_size=10, d_model=256, num_stage=12, dct_n=20, input_n=50):
        super(AttModel, self).__init__()

        opt = Options().parse()
        self.in_features = int(in_features/2)
        self.in_features = self.in_features  # remove this
        self.d_model = d_model
        self.dct_n = dct_n
        #change these
        # dct_n = 50
        # self.dct_n = dct_n
        #change these
        self.input_n = input_n
        self.kernel_size = kernel_size # to compute K_i
        self.chunk_size = 2 * kernel_size # to compute V_i
        # self.chunk_size = dct_n #remove this
        self.dim_xia_v = input_n - self.chunk_size + 1

        self.convQ1 = nn.Sequential(nn.Conv1d(in_channels=self.in_features, out_channels=d_model, kernel_size=1, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=6, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=5, bias=False),
                                   nn.ReLU())
        self.convK11 = nn.Sequential(nn.Conv1d(in_channels=self.in_features, out_channels=d_model, kernel_size=1, bias=False),
                                   nn.ReLU())
        self.convK12 = nn.Sequential(nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=6, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=5, bias=False),
                                   nn.ReLU())
        # self.gcn1 = GCN.GCN(input_feature=(dct_n) * 2, hidden_feature=d_model, p_dropout=0.3, num_stage=num_stage, node_n=self.in_features)
        self.gcn1 = GCN.GCN(input_feature=((dct_n)*2+7), hidden_feature=d_model, p_dropout=0.3, num_stage=num_stage, node_n=self.in_features)


        self.convQ2 = nn.Sequential(nn.Conv1d(in_channels=self.in_features, out_channels=d_model, kernel_size=1, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=6, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=5, bias=False),
                                   nn.ReLU())
        self.convK21 = nn.Sequential(nn.Conv1d(in_channels=self.in_features, out_channels=d_model, kernel_size=1, bias=False),
                                   nn.ReLU())
        self.convK22 = nn.Sequential(nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=6, bias=False),
                                   nn.ReLU(),
                                   nn.Conv1d(in_channels=d_model, out_channels=d_model, kernel_size=5, bias=False),
                                   nn.ReLU())
        # self.gcn2 = GCN.GCN(input_feature=(dct_n) * 2, hidden_feature=d_model, p_dropout=0.3, num_stage=num_stage, node_n=self.in_features)
        self.gcn2 = GCN.GCN(input_feature=((dct_n)*2+7), hidden_feature=d_model, p_dropout=0.3, num_stage=num_stage, node_n=self.in_features)
        self.fL = nn.Linear(126,108)
        self.s1 = nn.Linear(61,54)
        self.s2 = nn.Linear(61,54)
        # self.cL = nn.Linear(7,7)
        # self.s1 = nn.Linear(54,54)
        # self.s2 = nn.Linear(54,54)
        self.ig1 = nn.Linear(50,20)
        self.ig2 = nn.Linear(50,20)
        self.update_k1 = XIA.XIA(embed_dim=d_model, nb_h=8, dropout=0.1) # d_model = 256
        self.update_k2 = XIA.XIA(embed_dim=d_model, nb_h=8, dropout=0.1)
        self.update_v1 = XIA.XIA(embed_dim=self.in_features, nb_h=6, dropout=0.1) # in_features = 54
        self.update_v2 = XIA.XIA(embed_dim=self.in_features, nb_h=6, dropout=0.1)
        #uncomment for original
        self.src1 = nn.LSTM(54,54,batch_first=True)
        self.src2 = nn.LSTM(54,54,batch_first=True)
        self.l1 = nn.Linear(54,54)
        self.l2 = nn.Linear(54,54)
        self.actll = nn.Linear(7,7)
        self.src1dec = nn.LSTM(54*20,20,batch_first=True)
        self.src2dec = nn.LSTM(54*20,20,batch_first=True)

        self.d1 = nn.Linear(20,20)
        self.d2 = nn.Linear(20,20)

        self.ls1 = nn.LSTM(54,10,batch_first=True)
        self.ls2 = nn.LSTM(500,200,batch_first=True)
        self.ls1_ = nn.LSTM(54,10,batch_first=True)
        self.ls2_ = nn.LSTM(500,200,batch_first=True)
        self.dec1 = nn.Linear(200,54)
        self.dec2 = nn.Linear(200,54)
        self.dec1_ = TimeDistributed(self.dec1,batch_first=True)
        self.dec2_ = TimeDistributed(self.dec2,batch_first=True)
        self.sr1 = nn.Linear(54,54)
        self.sr2 = nn.Linear(54,54)
        # self.update_k1 = XIA.XIA(embed_dim=d_model, nb_h=5, dropout=0.1) # d_model = 256
        # self.update_k2 = XIA.XIA(embed_dim=d_model, nb_h=5, dropout=0.1)
        # self.update_v1 = XIA.XIA(embed_dim=self.in_features, nb_h=9, dropout=0.1) # in_features = 54
        # self.update_v2 = XIA.XIA(embed_dim=self.in_features, nb_h=9, dropout=0.1)

    def forward(self, src):
        # src1 = src[:,:,:self.in_features] # (bs, 50, 54)
        # src2 = src[:,:,self.in_features:] # (bs, 50, 54)

        src1 = src[:,:,:54] # (bs, 50, 54)
        src2 = src[:,:,54:108] # (bs, 50, 54)
        act = src[:,:,108]
        actonehot = torch.zeros(7)
        #         # print(act[i][j])
        # actonehot[int(act[0][0])]=1
        actonehot[int(act[0][0])]=1
        # print(actonehot)
        a , b , c = src1.shape
        actonehot = torch.reshape(actonehot,(1,1,7)).cuda()
        # print(actonehot.shape)
        actonehot = actonehot.repeat(a,54,1)
        # print(actonehot.shape)
        actonehot = torch.reshape(actonehot,(a,54,7))
        actonehot = self.actll(actonehot)
        # a , b , c = src.shape
        # # src1_ = torch.zeros((a,b,54+7))
        # # src2_ = torch.zeros((a,b,54+7))
        # src1_ = torch.zeros((a,b,54+7))
        # src2_ = torch.zeros((a,b,54+7))
        # for i in range(a):
        #     # print('hi')
        #     for j in range(b):
        #         # temp = nn.functional.one_hot(act[i][j],num_classes=7)
        #         # temp = torch.zeros(7)
        #         temp = torch.zeros(7)
        # #         # print(act[i][j])

        #         temp[int(act[i][j])]=1
        #         # print(temp)
        #         temp = temp.cuda()
        #         # temp = self.cL(temp)
        #         # print(temp)
        #         src1_[i][j] = torch.cat((src1[i][j],temp),0)
        #         src2_[i][j] = torch.cat((src2[i][j],temp),0)
        #         # print(src1[i][j].shape)
        #         # src1[i][j] = temp2
        #         # print(src1_[i][j].shape)
        #         # print("~~~~~")
        #         # print(src2_[i][j].shape)
        #         # print(temp2.shape)
        #         # print(temp)
        #         # print(act[i][j])
        #         # print(temp)
        # src1_ = src1_.cuda()
        # src2_ = src2_.cuda()
        # src1_ = self.s1(src1_)
        # src2_ = self.s2(src2_)

        # print(src1_.shape)
        # print(src2_.shape)
        # print("hello")
        # src1 = self.sr1(src1)
        # src2 = self.sr2(src2)
        src_ = [src1, src2]
        #comment added for lstm before attention
        # out, _ = self.src1(src1)
        # src1_ = out
        # src1_ = self.l1(src1_)
        # out2, _ = self.src2(src2)
        # src2_ = out2
        # src2_ = self.l2(src2_)
        # src_2 = [src1_, src2_]
        # print(src1_.shape)
        # print(src2_.shape)
        # print("SHAPES HERE")
        # print(self.in_features)
        dct_m, idct_m = util.get_dct_matrix(self.chunk_size) # (20, 20)
        dct_m = torch.from_numpy(dct_m).float().cuda()
        idct_m = torch.from_numpy(idct_m).float().cuda()

        dct_m2, idct_m2 = util.get_dct_matrix(60) # (20, 20)
        dct_m2 = torch.from_numpy(dct_m2).float().cuda()
        idct_m2 = torch.from_numpy(idct_m2).float().cuda()
        dct_n = self.dct_n
        vn = self.dim_xia_v
        vl = self.chunk_size

        idx_val = np.expand_dims(np.arange(vl), axis=0) + np.expand_dims(np.arange(vn), axis=1)# (31, 20)
        idx_key = np.expand_dims(np.arange(self.kernel_size), axis=0) + np.expand_dims(np.arange(vn), axis=1)# (31, 10)
        # print(idx_key)
        # print(idx_key.shape)
        # print("idx val and key shape here")
        src_tmp_, query_tmp_, key_tmp_, src_value_tmp_ = [None]*2,[None]*2,[None]*2,[None]*2

        for i in range(2):
            src = src_[i] #uncomment
            # src = src_2[i]
            src_tmp = src.clone()
            bs = src.shape[0]

            # k,q
            src_key_tmp = src_tmp.transpose(1, 2)[:, :, :(self.input_n - self.kernel_size)].clone() # (bs, 54, 40)
            src_query_tmp = src_tmp.transpose(1, 2)[:, :, -self.kernel_size:].clone() # (bs, 54, 10)
            if i == 0:
                key_tmp = self.convK11(src_key_tmp / 1000.0) # (bs, d_model, 40)
                query_tmp = self.convQ1(src_query_tmp / 1000.0) # (bs, d_model, 1)
            else:
                key_tmp = self.convK21(src_key_tmp / 1000.0)
                query_tmp = self.convQ2(src_query_tmp / 1000.0)
            # print(key_tmp.shape)    
            key_tmp = key_tmp[:,:,idx_key] # (bs, d_model, 31, 10)
            # print(query_tmp.shape)
            # print(key_tmp.shape)
            # print("shapes here")
            key_tmp = key_tmp.transpose(1,2).reshape(bs*vn, self.d_model, -1) # (bs*31, d_model, 10)
            query_tmp = query_tmp.transpose(1, 2) # (bs, 1, d_model)
            # v
            src_value_tmp = src_tmp[:, idx_val].clone().reshape([bs * vn, vl, -1]) # (bs, vn, vl, 54) -> (bs x vn, vl, 54)
            # print(src_value_tmp.shape)
            # print(dct_m.shape)
            # print(dct_m[:dct_n].unsqueeze(dim=0).shape)
            # print("shapes 2 here")
            src_value_tmp = torch.matmul(dct_m[:dct_n].unsqueeze(dim=0), src_value_tmp).transpose(1,2) # (bs x vn, 54, dct_n)
            # print(src_value_tmp.shape)
            src_tmp_[i], query_tmp_[i], key_tmp_[i], src_value_tmp_[i] = src_tmp, query_tmp, key_tmp, src_value_tmp

        ## update k v
        key_tmp_1 = self.update_k1(key_tmp_[0], key_tmp_[1]) # (bs*31, d_model, 10) : (batch_size, E, L)
        key_tmp_1 = self.convK12(key_tmp_1).squeeze() # (bs*31, d_model)
        key_tmp_1 = key_tmp_1.reshape(bs, vn, -1).permute(0, 2, 1) # (bs, d_model, 31)
        src_value_tmp_1 = self.update_v1(src_value_tmp_[0], src_value_tmp_[1]) # (bs*31, 54, dct_n): (batch_size, E, L)
        src_value_tmp_1 = src_value_tmp_1.reshape(bs, vn, -1) # (bs, 31, 54*dct_n)

        key_tmp_2 = self.update_k2(key_tmp_[1],key_tmp_[0])
        key_tmp_2 = self.convK22(key_tmp_2).squeeze()
        key_tmp_2 = key_tmp_2.reshape(bs, vn, -1).permute(0, 2, 1)
        src_value_tmp_2 = self.update_v2(src_value_tmp_[1], src_value_tmp_[0])
        src_value_tmp_2 = src_value_tmp_2.reshape(bs, vn, -1)

        src_tmp_1, query_tmp_1, src_tmp_2, query_tmp_2 = src_tmp_[0], query_tmp_[0], src_tmp_[1], query_tmp_[1]

        score_tmp_1 = torch.matmul(query_tmp_1, key_tmp_1) + 1e-15 # (bs, 1, d_model) x (bs, d_model, 31)
        att_tmp_1 = score_tmp_1 / (torch.sum(score_tmp_1, dim=2, keepdim=True)) # (bs, 1, 31)
        dct_att_tmp_1 = torch.matmul(att_tmp_1, src_value_tmp_1)[:, 0].reshape([bs, -1, dct_n]) # (bs, 54, 20)

        score_tmp_2 = torch.matmul(query_tmp_2, key_tmp_2) + 1e-15
        att_tmp_2 = score_tmp_2 / (torch.sum(score_tmp_2, dim=2, keepdim=True))
        dct_att_tmp_2 = torch.matmul(att_tmp_2, src_value_tmp_2)[:, 0].reshape([bs, -1, dct_n])


        # gcn
        idx = list(range(-self.kernel_size, 0, 1)) + [-1] * self.kernel_size
        # idx = list(range(-self.kernel_size*5, 0, 1)) + [-1] * self.kernel_size
        # idx = list(range(-(2*self.kernel_size),0,1))
        # print(src_tmp_1.shape)
        # idx2 = [-1] * self.kernel_size
        # print("SRC TMP SHAPE")
        # a , b , c = src_tmp_1.shape
        # x , y , z = dct_att_tmp_1.shape
        # # print(dct_att_tmp_1.shape)
        # # print("xyz here")
        # tmpd1 = torch.reshape(dct_att_tmp_1,(x,1,y*z))
        # # print(type(tmpd1))
        # tmpd1 = tmpd1.repeat(1,1,54)
        # tmpd1 = torch.reshape(tmpd1,(x,54,y*z))
        # # print(type(tmpd1))
        # outdec1 , _ = self.src1dec(tmpd1)
        # # print(outdec1.shape)
        # outdec1 = self.d1(outdec1)
        # # print(type(outdec1))
        # dct_att_tmp_1 = outdec1
        # print(type(dct_att_tmp_1))
        
        # out, sout = self.ls1(src_tmp_1)
        # # print(out.shape)
        # # inputdec = sout[0]
        # inputdec = out
        # x , y , z = inputdec.shape
        # inputdec = torch.reshape(inputdec,(x,y*z))
        # # print(inputdec.shape)
        # # print("inputdec here")
        # # print(type(inputdec))
        # # print(inputdec.grad_fn)
        # # print("input dec here")
        # inputdec = torch.as_tensor(inputdec)
        # # print(inputdec.shape)
        # # print("OUT SHAPE HE RE")
        # temp1 = src_tmp_1[:,-1]
        # temp1 = torch.reshape(temp1,(1,a,54))
        # temp1 = temp1.repeat(1,1,10)
        # temp1 = torch.reshape(temp1,(a,10,54))
        # # # restemp = torch.cat([inputdec,temp1],axis=-1)
        # # # print(restemp.shape)
        # # # print("restemp shape here")
        # # # res = inputdec.repeat(1,1,20)
        # res = inputdec.repeat(1,1,10)
        # # print(res.shape)
        # res = res.reshape(a,10,500)
        # # print(res.shape)
        # outdec,__ = self.ls2(res)
        # outdec = self.dec1(outdec)
        # outdec = torch.cat([outdec,temp1],axis=1)
        # # print(outdec.shape)
        # # print('shape here')
        # # print(outdec.shape)
        # # print("DEC SIZE HERE")
        # input_gcn_1 = outdec
    
        # print(dct_att_tmp_1.shape)
        # print(dct_att_tmp_2.shape)
        # print("DCT ATT SHAPE HERE")
        input_gcn_1 = src_tmp_1[:, idx] #important over here
        # input_gcn_1 = self.ig1(src_tmp_1)
        # input_gcn_1 = src_tmp_1[:,30:]
        # input_gcn_1 = src_tmp_1
        # print(input_gcn_1.shape)
        # print("GCN shapes here")
        # print(len(idx))
        # print(idx)
        # print("IDX HERE")
        # print(input_gcn_1.shape)
        # print("gcn1 here")
        dct_in_tmp_1 = torch.matmul(dct_m[:dct_n].unsqueeze(dim=0), input_gcn_1).transpose(1, 2)
        # dct_in_tmp_1 = torch.matmul(dct_m2[:60].unsqueeze(dim=0), input_gcn_1).transpose(1, 2)
        # print(dct_in_tmp_1.shape)
        # # print("shape 1 here")
        # actonehot = torch.reshape(a,1,7)
        # actonehot = actonehot.repeat(1,54,1)
        # actonehot = torch.reshape(act,(a,54,7))
        dct_in_tmp_1 = torch.cat([dct_in_tmp_1, dct_att_tmp_1], dim=-1) # bs x 54 x 20 
        #appending one hot
        dct_in_tmp_1 = torch.cat([dct_in_tmp_1,actonehot],dim=-1)
        # dct_in_tmp_1 = dct_att_tmp_1
        # print(dct_in_tmp_1.shape)
        # print("shape 2 here")
        input_gcn_2 = src_tmp_2[:, idx]

        # x , y , z = dct_att_tmp_2.shape
        # tmpd2 = torch.reshape(dct_att_tmp_2,(x,1,y*z))
        # tmpd2 = tmpd2.repeat(1,1,54)
        # tmpd2 = torch.reshape(tmpd2,(x,54,y*z))
        # outdec2 , _ = self.src2dec(tmpd2)
        # outdec2 = self.d2(outdec2)
        # dct_att_tmp_2 = outdec2

        # print(input_gcn_2.shape)
        # input_gcn_2 = self.ig2(src_tmp_2)
        # out2, sout2 = self.ls1_(src_tmp_2)
        # # print(out.shape)
        # inputdec2 = out2
        # x2 , y2 , z2 = inputdec2.shape
        # inputdec2 = torch.reshape(inputdec2,(x2,y2*z2))
        # # inputdec2 = sout2[0]
        # inputdec2 = torch.as_tensor(inputdec2)
        # # print(inputdec.shape)
        # # print("OUT SHAPE HERE")
        # temp2 = src_tmp_2[:,-1]
        # temp2 = torch.reshape(temp2,(1,a,54))
        # temp2 = temp2.repeat(1,1,10)
        # temp2 = torch.reshape(temp2,(a,10,54))
        # # restemp2 = torch.cat([inputdec2,temp2],axis=-1)
        # res2 = inputdec2.repeat(1,1,10)
        # # print(res.shape)
        # res2 = res2.reshape(a,10,500)
        # # print(res.shape)
        # outdec2,__2 = self.ls2_(res2)
        # outdec2 = self.dec2(outdec2)
        # outdec2 = torch.cat([outdec2,temp2],axis=1)
        # # print(outdec2.shape)
        # # print('shape2 here')
        # # print(outdec.shape)   
        # # print("DEC SIZE HERE")
        # input_gcn_2 = outdec2
        # input_gcn_2 = src_tmp_2[:,30:]
        # input_gcn_2 = src_tmp_2
        # print(input_gcn_2.shape)
        # print("GCN Shape here")
        dct_in_tmp_2 = torch.matmul(dct_m[:dct_n].unsqueeze(dim=0), input_gcn_2).transpose(1, 2)
        # dct_in_tmp_2 = torch.matmul(dct_m2[:60].unsqueeze(dim=0), input_gcn_2).transpose(1, 2)
        dct_in_tmp_2 = torch.cat([dct_in_tmp_2, dct_att_tmp_2], dim=-1)
        dct_in_tmp_2 = torch.cat([dct_in_tmp_2,actonehot],dim=-1)
        # dct_in_tmp_2 = dct_att_tmp_2
        # print(input_gcn_1.shape)
        # print(input_gcn_2.shape)
        # print("GCN shapes here")
        # print(dct_att_tmp_1.shape)
        # print(dct_att_tmp_2.shape)
        # print("DCT ATT SHAPE HERE")
        # print(idct_m.shape)
        # print(dct_in_tmp_1.shape)
        # print(dct_in_tmp_2.shape)
        # print("~~~DCT HERE IN ~~")
        dct_out_tmp_1 = self.gcn1(dct_in_tmp_1)
        # print(dct_out_tmp_1.shape)
        # print("gcn shape 1 here")
        # print(idct_m.shape)
        # print("idct shape here")
        dct_out_tmp_2 = self.gcn2(dct_in_tmp_2)
        # print(dct_out_tmp_1.shape)
        # print(dct_out_tmp_2.shape)
        # print("DCT SHAPE HERE")
        # print(idct_m2.shape)
        # idct
        out_gcn_1 = torch.matmul(idct_m[:, :dct_n].unsqueeze(dim=0), dct_out_tmp_1[:, :, :dct_n].transpose(1, 2))
        out_gcn_2 = torch.matmul(idct_m[:, :dct_n].unsqueeze(dim=0), dct_out_tmp_2[:, :, :dct_n].transpose(1, 2))
        # out_gcn_1 = torch.matmul(idct_m2[:dct_n:, :dct_n].unsqueeze(dim=0), dct_out_tmp_1[:, :, :dct_n].transpose(1, 2))
        # out_gcn_2 = torch.matmul(idct_m2[:dct_n:, :dct_n].unsqueeze(dim=0), dct_out_tmp_2[:, :, :dct_n].transpose(1, 2))
        # print(out_gcn_1.shape)
        # print(out_gcn_2.shape)
        # print("shapes here")

        # out, sout = self.ls1(out_gcn_1)
        # # print(out.shape)
        # inputdec = sout[0]
        # inputdec = torch.as_tensor(inputdec)
        # # print(inputdec.shape)
        # # print("OUT SHAPE HERE")
        # res = inputdec.repeat(1,1,20)
        # # print(res.shape)
        # res = res.reshape(a,20,300)
        # # print(res.shape)
        # outdec,__ = self.ls2(res)
        # outdec = self.dec1(outdec)


        # # input_gcn_2 = self.ig2(src_tmp_2)
        # out2, sout2 = self.ls1_(out_gcn_2)
        # # print(out.shape)
        # inputdec2 = sout2[0]
        # inputdec2 = torch.as_tensor(inputdec2)
        # # print(inputdec.shape)
        # # print("OUT SHAPE HERE")
        # res2 = inputdec2.repeat(1,1,20)
        # # print(res.shape)
        # res2 = res2.reshape(a,20,300)
        # # print(res.shape)
        # outdec2,__2 = self.ls2_(res2)
        # outdec2 = self.dec2(outdec2)
        # print(outdec.shape)   
        # print("DEC SIZE HERE")
        # input_gcn_2 = outdec2
        # print(outdec.shape)
        # print("DEC SIZE HERE")
        
        # outputs = torch.cat([outdec,outdec2],axis=2).unsqueeze(2)
        outputs = torch.cat([out_gcn_1,out_gcn_2], axis=2).unsqueeze(2)
        # outputs_ = self.fL(outputs)
        # print(outputs_.shape)self.TD
        # print(outputs.shape)
        # print("OUTPUT SHAPE HERE")
        return outputs

if __name__ == '__main__':
    p3d_src = torch.rand((32,75,108)).cuda()
    net_pred = AttModel(in_features=108,
            kernel_size=10,
            d_model=256,
            num_stage=12,
            dct_n=20).cuda().eval()
    p3d_out_all = net_pred(p3d_src)


