'''
Software ExPI
Copyright Inria
Year 2021
Contact : wen.guo@inria.fr
GPL license.
'''
# data_utils.py
# func utils for data

import numpy as np
import torch
# from IPython import embed

###########################################
## func for reading data

def readCSVasFloat(filename, with_key=True):
    """
    Borrowed from SRNN code. Reads a csv and returns a float matrix.
    https://github.com/asheshjain399/NeuralModels/blob/master/neuralmodels/utils.py#L34
    Args
      filename: string. Path to the csv file
    Returns
      returnArray: the read data in a float32 matrix
    """
    returnArray = []
    lines = open(filename).readlines()
    if with_key: # skip first line
        lines = lines[1:]
    for line in lines:
        line = line.strip().split(',')
        if len(line) > 0:
            returnArray.append(np.array([np.float32(x) for x in line]))

    returnArray = np.array(returnArray)
    return returnArray

###########################################
## func utils for norm/unnorm

def normExPI_xoz(img, P0,P1,P2):
    # P0: orig
    # P0-P1: axis x
    # P0-P1-P2: olane xoz

    X0 = P0
    X1 = (P1-P0) / np.linalg.norm((P1-P0)) + P0 #x
    X2 = (P2-P0) / np.linalg.norm((P2-P0)) + P0
    X3 = np.cross(X2-P0, X1-P0) + P0 #y
    ### x2 determine z -> x2 determine plane xoz
    X2 = np.cross(X1-P0, X3-P0) + P0 #z

    X = np.concatenate((np.array([X0,X1,X2,X3]).transpose(), np.array([[1, 1, 1,1]])), axis = 0)
    Q = np.array([[0,0,0],[1,0,0],[0,0,1], [0,1,0]]).transpose()
    M  = Q.dot(np.linalg.pinv(X))
    
    # print(M.shape,"M SHAPE")

    # UN COMMENT FROM HERE~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Msquare = np.concatenate([M,np.array([0,0,0,1]).reshape(1,4)])
    # # print(M)
    # # print(Msquare)
    # Msquareinv = np.linalg.pinv(Msquare)
    # Msquareinv = Msquareinv.reshape(1,16)
    # Msquareinv = np.concatenate([Msquareinv,np.array([0,0]).reshape(1,2)],axis=1).reshape(6,3)
    #~~~~~~~~~~~~~~~~~~~~~~
    # exit()
    # exit()
    # print(M.T,M.T.shape,"MT SHAPE")
    # Mtemp = M.T
    # M_inv = np.linalg.pinv(M)
    # # print(M.shape,"M SHAPE HERE")
    # print(M_inv.shape,"M_INV SHAPE")
    # # print((M.T).dot(M_inv))
    # print(M_inv.dot(M))
    # exit()
    # # print(M_inv,"MINV HERE")

    # temp = np.array([1,1,1,1]).reshape(1,4)
    # # print(M.shape,temp.shape)
    # M_ = np.concatenate([M,temp],axis=0)
    # M_inv = np.linalg.pinv(M_)
    # # print(M_inv,"MINV HERE")
    # # exit()
    # M_inv = M_inv.reshape(1,16)
    # M_inv = np.concatenate([M_inv,np.array([0,0]).reshape(1,2)],axis=1).reshape(6,3)
    # print(M_inv.shape)
    # print(M_inv)
    # M_ = np.concatenate([M,np.array([1,1,1,1])],axis=0)


    #code i added#
    # print(M.shape,'M SHAPE')
    # Minv = np.linalg.inv(M)
    # print(M.shape,Minv.shape,"M AND MINV HERE")
    img_norm = img.copy()
    for i in range(len(img)):
        tmp = img[i]
        # print(tmp)
        # print(tmp.shape,"TMP SHAPE FIRST")
        tmp = np.concatenate((tmp,np.array([1])),axis=0)
        # print(tmp)
        # print(tmp.shape,"TMP SHAPE NOW")
        img_norm[i] =  M.dot(tmp)

        # print(img_norm[i].shape,img_norm[i],"AFTER MULTIPLY")
        # temp1 = M.dot(tmp)
        # # print(temp1,"TEMP1")
        # temp1 = np.concatenate([temp1,np.array([1])]).reshape(4,1)
        # print(temp1,"TEMP1 NOW")
        # temp1inv = Msquareinv.dot(temp1)
        # print(temp1inv,"temp1inv values")

        # oldtemp1 = temp1.dot(Msquareinv)
        # print(oldtemp1,"OLD TEMP 1 NOW")
        # print(M,"M SHAPE VALUE HERE")
        # print(M_,"M_ SHAPE HERE")
        # print(M.dot(tmp),"IMG NORM SHAPE")
        # print(M.dot(tmp).shape,"IMG NORM SHAPE")
        # temp1 = M.dot(tmp)
        # print(temp1.shape,"TEMP 1 SHAPE")
        # temp1inv = temp1.dot(M_inv.transpose(1,0))
        # print(temp1inv,"temp1inv")
        # temp2 = M_.dot(tmp)
        # temp3 = np.concatenate([temp1,np.array([1])])
        # print(temp3,"TEMP3 HERE")
        # print(temp1,"TEMP 1 HERE")
        # print(temp2,"TEMP 2 HERE")
        # temp2_ = M_inv.dot(temp2)
        # temp3_ = M_inv.dot(temp3)
        # print(temp2_,"TEMP 2_ VALUE HERE")
        # print(temp3_,"TEMP3_ VALUE HERE")
    # print(img_norm.shape,"IMG NORM SHAPE")
    # print(M_inv.shape,"M_inv shape")
    # img_norm = np.concatenate([img_norm,Msquareinv])
    # print(img_norm.shape,"IMG NORM NEW SHAPE")
    return img_norm 

def normExPI_2p_by_frame(seq):
    nb, dim = seq.shape # nb_frames, dim=108
    seq_norm = seq.copy()
    # seq_norm = np.zeros((nb,dim+6*3))
    for i in range(nb):
        img = seq[i].reshape((-1,3)) #36
        P0 = (img[10] + img[11])/2
        P1 = img[11]
        P2 = img[3]
        img_norm = normExPI_xoz(img, P0,P1,P2)
        # dimnew = dim + 6*3
        # seq_norm[i] = img_norm.reshape(dimnew)
        seq_norm[i] = img_norm.reshape(dim)
    return seq_norm

def normExPI_2p_by_frameCHI3D(seq):
    nb, dim = seq.shape # nb_frames, dim=108
    seq_norm = seq.copy()
    # seq_norm = np.zeros((nb,dim+6*3))
    for i in range(nb):
        img = seq[i].reshape((-1,3)) #36
        P0 = (img[1] + img[4])/2
        P1 = img[4]
        P2 = img[8]
        img_norm = normExPI_xoz(img, P0,P1,P2)
        # dimnew = dim + 6*3
        # seq_norm[i] = img_norm.reshape(dimnew)
        seq_norm[i] = img_norm.reshape(dim)
    return seq_norm

def normExPI_2p_by_frameCMU(seq):
    nb, dim = seq.shape # nb_frames, dim=108
    seq_norm = seq.copy()
    for i in range(nb):
        img = seq[i].reshape((-1,3)) #36
        # P0 = (img[10] + img[11])/2 # left hip and right hip
        P0 = (img[1]+img[5])/2 # left thi and right thigh mid
        P1 = img[5] # right thigh
        P2 = img[10] # chest
        img_norm = normExPI_xoz(img, P0,P1,P2)
        seq_norm[i] = img_norm.reshape(dim)
    return seq_norm

def normExPI_2p_by_frameB(seq):
    nb, dim = seq.shape # nb_frames, dim=108
    seq_orig = seq.copy()
    seq_norm = seq.copy()
    seq_norm = np.concatenate([seq_norm[:,18*3:],seq_norm[:,:18*3]],axis=-1)
    seq = seq_norm
    for i in range(nb):
        img = seq[i].reshape((-1,3)) #36
        img2 = seq_orig[i].reshape((-1,3))
        P0 = (img[10] + img[11])/2
        P1 = img[11]
        P2 = img[3]
        img_norm = normExPI_xoz(img2, P0,P1,P2)
        seq_norm[i] = img_norm.reshape(dim)
    return seq_norm

def unnorm_abs2Indep(seq):
    # in:  torch.size(bz, nb_frames, 36, 3)
    # out: torch.size(bz, nb_frames, 36, 3)
    seq = seq.detach().cpu().numpy()
    bz, frame, nb, dim = seq.shape
    seq_norm = seq
    for j in range(bz):
        for i in range(frame):
            img = seq[j][i]

            P0_m = (img[10] + img[11])/2
            P1_m = img[11]
            P2_m = img[3]
            if nb == 36:
                img_norm_m = normExPI_xoz(img[:int(nb/2)], P0_m,P1_m,P2_m)
                P0_f = (img[18+10] + img[18+11])/2
                P1_f = img[18+11]
                P2_f = img[18+3]
                img_norm_f = normExPI_xoz(img[int(nb/2):], P0_f,P1_f,P2_f)
                img_norm = np.concatenate((img_norm_m, img_norm_f))
            elif nb == 18:
                img_norm = normExPI_xoz(img, P0_m,P1_m,P2_m)
            seq_norm[j][i] = img_norm.reshape((nb,dim))
    seq = torch.from_numpy(seq_norm).cuda()
    return seq


###########################################
## func utils for finding test samples

def find_indices_64(num_frames, seq_len):
    # not random choose. as the sequence is short and we want the test set to represent the seq better
    seed = 1234567890
    np.random.seed(seed)

    T = num_frames - seq_len + 1
    n = int(T / 64)
    list0 = np.arange(0,T)
    list1 = np.arange(0,T,(n+1))
    t =  64 - len(list1)
    if t == 0:
        listf = list1
    else:
        list2 = np.setdiff1d(list0, list1)
        list2 = list2[:t]
        listf = np.concatenate((list1, list2))
    return listf


